/* GluNNewQuadricOpenGLView */

#import <Cocoa/Cocoa.h>
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>

#define kTextureCount 10


typedef enum _QuadricType 
{	
	drawGluCylinder		= 0,
	drawGluDisk,
	drawGluPartialDisk,
	drawGluSphere	
} QuadricType;


typedef enum _QuadricNormalType 
{	
	useGLU_NONE		= 0,
	useGLU_FLAT,
	useGLU_SMOOTH
} QuadricNormalType;


typedef enum _QuadricOrientationType 
{	
	useGLU_OUTSIDE		= 0,
	useGLU_INSIDE	
} QuadricOrientationType;


typedef enum _CullFaceType 
{	
	disableCullFAce		= 0,
	doGL_FRONT,
	doGL_BACK,
	doGL_FRONT_AND_BACK
} CullFaceType;


typedef enum _BackgroundType 
{	
	blackBackground		= 0,
	paleRed,
	paleGreen,
	paleBlue,
	skyBlue,
	whiteBackground	
} BackgroundType;


@class StringRenderer;
@interface GluNNewQuadricOpenGLView : NSOpenGLView
{
	BOOL						inited;
	BOOL						optionKeyDown, controlKeyDown, commandKeyDown, shiftDown, capsLockDown;
	BOOL						drawGrid, drawAxes, drawEquatorialPlane, drawOrbitalPlane;
	BOOL						depthTestEnabled;
	long						swap_enabled;
	GLdouble					minXOrtho, maxXOrtho, minYOrtho, maxYOrtho, minZOrtho, maxZOrtho;
	GLdouble					glOrthoFactor;
	GLdouble					eyeX,  eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ;
	QuadricType					quadric;
	QuadricNormalType			quadricNormal;
	QuadricOrientationType		quadricOrientation;
	CullFaceType				cullFace;
	GLUquadric					*quad;
	BackgroundType				background;
	GLdouble					base, top, height, radius;
	GLdouble					inner, outer, start, sweep;
	GLdouble					zRotation, dZRotation;
	GLint						loops, slices, stacks;
	GLuint						textures[kTextureCount];
	GLint						currentTexture;
	GLenum						drawStyle;
	NSTimeInterval				zRotationInterval;
	NSTimer						*zRotationTimer;
	StringRenderer				*stringRenderer;
	NSArray						*textureLabelsArray;
}
#pragma mark initialization and deallocation
- (void)delayedInit;

- (void)initTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
   forPictureAtPath:(NSString *)path
			 ofType:(NSString *)type;

- (void)initCheckeredTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
					colorOne:(NSColor *)colorOne
					colorTwo:(NSColor *)colorTwo
			horizontalBlocks:(int)horizontalBlocks
			  verticalBlocks:(int)verticalBlocks;

- (void)initStripedTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
					colorOne:(NSColor *)colorOne
					colorTwo:(NSColor *)colorTwo
				   stripes:(int)stripes;

- (void)textureInitCommonForTexture:(GLuint)_texture
							  level:(GLint)level
					 internalformat:(GLint)internalformat
							  width:(GLsizei)width
							 height:(GLsizei)height
							 border:(GLint)border
							 format:(GLenum)format
							   type:(GLenum)type
							 pixels:(const GLvoid *)pixels;

#pragma mark drawing
- (void)drawAxes;
- (void)drawQuadric;
- (void)drawCylinder;
- (void)drawDisk;
- (void)drawPartialDisk;
- (void)drawSphere;
- (void)drawTextureInRectangle;
- (void)drawVariablesLabels;
- (void)drawCullFaceLabel;
- (void)drawDrawStyleLabel;
- (void)drawNormalsLabel;
- (void)drawOrientationLabel;
- (void)drawDepthTestEnabledLabel;
- (void)drawQuadricLabel;
- (void)drawTextureLabel;
- (void)drawZRotationLabel;
- (void)setupInitialCoordinateSystem;
- (void)setupLabelCoordinates;
#pragma mark labels
- (char *)cullFaceLabel;
- (char *)drawStyleLabel;
- (char *)depthTestEnabledLabel;
- (char *)textureLabel;
- (char *)zRotationLabel;
#pragma mark control
- (void)updateCullFace:(NSEvent *)theEvent;
- (void)updateDrawStyle:(NSEvent *)theEvent;
- (void)updateNormals:(NSEvent *)theEvent;
- (void)updateOrientation:(NSEvent *)theEvent;
- (void)updateQuadric:(NSEvent *)theEvent;
- (void)updateBackground:(NSEvent *)theEvent;
- (void)updateCurrentTexture:(NSEvent *)theEvent;
- (void)startZRotationTimer;
- (void)stopZRotationTimer;
- (void)animateZRotation:(NSTimer *)timer;
- (void)toggleZRotationAnimation:(NSEvent *)theEvent;
- (void)toggleDepthTestEnabled:(NSEvent *)theEvent;
- (void)toggleDrawAxes:(NSEvent *)theEvent;
#pragma mark accessors
- (void)setBase:(GLdouble)_base;
- (void)setTop:(GLdouble)_top;
- (void)setHeight:(GLdouble)_height;
- (void)setRadius:(GLdouble)_radius;
- (void)setInner:(GLdouble)_inner;
- (void)setOuter:(GLdouble)_outer;
- (void)setStart:(GLdouble)_start;
- (void)setSweep:(GLdouble)_sweep;
- (void)setLoops:(GLint)_loops;
- (void)setSlices:(GLint)_slices;
- (void)setStacks:(GLint)_stacks;
@end
