#import "GluNNewQuadricOpenGLView.h"
#import "StringRenderer.h"

void printTileGLError(char *label, GLenum error)
{
	char *errStr;
	
	switch (error)
	{
		case GL_NO_ERROR:
			errStr = "GL_NO_ERROR";
			break;
			
		case GL_INVALID_ENUM:
			errStr = "GL_INVALID_ENUM";
			break;
			
		case GL_INVALID_VALUE:
			errStr = "GL_INVALID_VALUE";
			break;
			
		case GL_INVALID_OPERATION:
			errStr = "GL_INVALID_OPERATION";
			break;
			
		case GL_STACK_OVERFLOW:
			errStr = "GL_STACK_OVERFLOW";
			break;
			
		case GL_STACK_UNDERFLOW:
			errStr = "GL_STACK_UNDERFLOW";
			break;
			
		case GL_OUT_OF_MEMORY:
			errStr = "GL_OUT_OF_MEMORY";
			break;
			
		default: errStr = "undefined error";
	}
	printf("%s%s\n", label, errStr);
}

GLenum pixelTypeForBPS(unsigned int bitsPerSample)
{
	GLenum pixelTypeForBPS = GL_UNSIGNED_BYTE;		// default to GL_UNSIGNED_BYTE
	switch(bitsPerSample)
	{
		// case 8:
			// pixelTypeForBPS =  GL_UNSIGNED_BYTE;
		
		case 16:
			pixelTypeForBPS = GL_UNSIGNED_SHORT;
			break;
			
		case 32:
			pixelTypeForBPS = GL_UNSIGNED_INT;
			break;
	}
	return pixelTypeForBPS; 
}


GLint internalFormatForColorspace(NSString *_colorSpaceName, BOOL _imageHasAlpha)
{
	if ([_colorSpaceName isEqualToString:NSCalibratedRGBColorSpace])		return _imageHasAlpha ? GL_RGBA : GL_RGB;
	if ([_colorSpaceName isEqualToString:NSDeviceRGBColorSpace])			return _imageHasAlpha ? GL_RGBA : GL_RGB;
	
	if ([_colorSpaceName isEqualToString:NSCalibratedWhiteColorSpace])		return _imageHasAlpha ? GL_LUMINANCE_ALPHA : GL_LUMINANCE;
	if ([_colorSpaceName isEqualToString:NSDeviceWhiteColorSpace])			return _imageHasAlpha ? GL_LUMINANCE_ALPHA : GL_LUMINANCE;
	
	// if ([_colorSpaceName isEqualToString:NSCalibratedBlackColorSpace])	return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSDeviceBlackColorSpace])		return _imageHasAlpha ? : ;
	
	// if ([_colorSpaceName isEqualToString:NSDeviceCMYKColorSpace])		return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSNamedColorSpace])				return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSPatternColorSpace])			return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSCustomColorSpace])			return _imageHasAlpha ? : ;
	
	return GL_RGBA;
}

// 	if ([_colorSpaceName isEqualToString:]) return _imageHasAlpha ? : ;


GLint formatForColorspace(NSString *_colorSpaceName, BOOL _imageHasAlpha)
{
	if ([_colorSpaceName isEqualToString:NSCalibratedRGBColorSpace])		return _imageHasAlpha ? GL_RGBA : GL_RGB;
	if ([_colorSpaceName isEqualToString:NSDeviceRGBColorSpace])			return _imageHasAlpha ? GL_RGBA : GL_RGB;
	
	if ([_colorSpaceName isEqualToString:NSCalibratedWhiteColorSpace])		return _imageHasAlpha ? GL_LUMINANCE_ALPHA : GL_LUMINANCE;
	if ([_colorSpaceName isEqualToString:NSDeviceWhiteColorSpace])			return _imageHasAlpha ? GL_LUMINANCE_ALPHA : GL_LUMINANCE;
	
	// if ([_colorSpaceName isEqualToString:NSCalibratedBlackColorSpace])	return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSDeviceBlackColorSpace])		return _imageHasAlpha ? : ;
	
	// if ([_colorSpaceName isEqualToString:NSDeviceCMYKColorSpace])		return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSNamedColorSpace])				return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSPatternColorSpace])			return _imageHasAlpha ? : ;
	// if ([_colorSpaceName isEqualToString:NSCustomColorSpace])			return _imageHasAlpha ? : ;

	return GL_RGBA;
}


// Thanks are due to Brian Nenniger for publishing Assimilator as open source code. Nenninger's generosity
// educated me on the need for image flipping and saved time in writing my own image flipping routine.

void flipImage(unsigned char *image, unsigned int rows, unsigned int calculatedBytesPerRow)
{
	void *rowBuffer = calloc(calculatedBytesPerRow, 1);
	unsigned int i, halfRowCount;
	unsigned char *sPtr, *dPtr;
	
	sPtr = image;
	dPtr = (rows - 1) * calculatedBytesPerRow + sPtr;
	
	halfRowCount = rows / 2;
	for (i = 0; i < halfRowCount; i++)
	{
		memcpy(rowBuffer, dPtr, calculatedBytesPerRow);
		memcpy(dPtr, sPtr, calculatedBytesPerRow);
		memcpy(sPtr, rowBuffer, calculatedBytesPerRow);
		sPtr += calculatedBytesPerRow;
		dPtr -= calculatedBytesPerRow;
	}
	
	free(rowBuffer);
}


void addColorToBufferAtIndex(char _red, 
							  char _green, 
							  char _blue, 
							  char _alpha, 
							  unsigned char *_buffer, 
							  int *_index)
{
	_buffer[(*_index)++] = _red;
	_buffer[(*_index)++] = _green;
	_buffer[(*_index)++] = _blue;
	_buffer[(*_index)++] = _alpha;
}


void setClearColorForBackground(BackgroundType _background)
{
	if (_background == blackBackground)		glClearColor(0.0, 0.0, 0.0, 1.0);
	if (_background == paleRed)				glClearColor(1.0, 0.7, 0.7, 1.0);
	if (_background == paleGreen)			glClearColor(0.7, 1.0, 0.7, 1.0);
	if (_background == paleBlue)			glClearColor(0.7, 0.7, 1.0, 1.0);
	if (_background == skyBlue)				glClearColor(0.6, 1.0, 1.0, 1.0);
	if (_background == whiteBackground)		glClearColor(1.0, 1.0, 1.0, 1.0);
}

@implementation GluNNewQuadricOpenGLView

- (id)initWithFrame:(NSRect)frameRect
		pixelFormat:(NSOpenGLPixelFormat*)format
{
    self = [super initWithFrame:frameRect pixelFormat:format];
	if (self)
	{
		// [self initFromDefaults];
		
		stringRenderer = [[StringRenderer alloc] initWithView:self];
		textureLabelsArray = [[NSArray arrayWithObjects:
			@"striped cyan magenta", 
			@"checkered red blue", 
			@"striped green yellow", 
			@"checkered orange purple", 
			@"DSC-040504-00044.w2",
			@"DSC03555.w", 
			@"M101", 
			@"land_shallow_topo_2048", 
			@"Lat Lon", 
			@"texture-coord-sphere",
			nil] retain];
		
		drawAxes = YES;
		depthTestEnabled = YES;
		
		base = 6000.0;
		top = 6000.0;
		height = 7000.0;
		radius = 7000.0;
		slices = 128;
		stacks = 128;
		inner = 1000.0;
		outer = 7000.0;
		loops = 5;
		start = 20.0;
		sweep = 320.0;
		
		drawStyle = GLU_FILL;
		
		currentTexture = 7;							// land_shallow_topo_2048
		
		quadric =				drawGluSphere;
		quadricNormal =			useGLU_SMOOTH;
		quadricOrientation =	useGLU_OUTSIDE;
		
		background =			blackBackground;
		
		
		zRotationInterval = 1.0 / 30.0;				// 1/30 second
		dZRotation = 36.0 * zRotationInterval;		// 360 every 10 seconds
	}
	return self;
}


- (id)initWithFrame:(NSRect)frameRect
{
	// printf("-[%s initWithFrame:]\n", [[self description] UTF8String]);
	
    NSOpenGLPixelFormatAttribute attrs[] =
    {
        NSOpenGLPFADoubleBuffer,
        NSOpenGLPFADepthSize, (NSOpenGLPixelFormatAttribute)32,
        (NSOpenGLPixelFormatAttribute)nil
    };
    NSOpenGLPixelFormat* pixFmt = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attrs] autorelease];
 	
	return [self initWithFrame:frameRect pixelFormat:pixFmt];
}


- (void)dealloc
{
	if (textures[0]) glDeleteTextures(kTextureCount, textures);
	[textureLabelsArray release];
	[stringRenderer release];
	
	[super dealloc];
}



- (BOOL)isFlipped
{
	return YES;
}



- (void)delayedInit
{
	[[self openGLContext] makeCurrentContext];
	
	swap_enabled = 1;
	[[self openGLContext] setValues:&swap_enabled forParameter:NSOpenGLCPSwapInterval];
	
	if (depthTestEnabled) glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	
	
	// glEnable(GL_BLEND);
	// glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
	// glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	setClearColorForBackground(background);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	
	glEnable(GL_TEXTURE_2D);
	// glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
	glGenTextures(kTextureCount, textures);
	
	
	if (textures[0]) [self initStripedTexture:textures[0]
									   sizedS:1024
										  byT:1024
									 colorOne:[NSColor cyanColor]
									 colorTwo:[NSColor magentaColor]
									  stripes:16];
	
	if (textures[1]) [self initCheckeredTexture:textures[1]
										 sizedS:512
											byT:512
									   colorOne:[NSColor redColor]
									   colorTwo:[NSColor blueColor]
							   horizontalBlocks:8
								 verticalBlocks:8];
	
	if (textures[2]) [self initStripedTexture:textures[2]
									   sizedS:1024
										  byT:1024
									 colorOne:[NSColor greenColor]
									 colorTwo:[NSColor yellowColor]
									  stripes:16];
	
	if (textures[3]) [self initCheckeredTexture:textures[3]
										 sizedS:128
											byT:128
									   colorOne:[NSColor orangeColor]
									   colorTwo:[NSColor purpleColor]
							   horizontalBlocks:8
								 verticalBlocks:8];
	
	if (textures[4]) [self initTexture:textures[4]
								sizedS:2048
								   byT:1024
					  forPictureAtPath:@"DSC-040504-00044.w2"
								ofType:@"jpg"];
	
	if (textures[5]) [self initTexture:textures[5]
								sizedS:4096
								   byT:2048
					  forPictureAtPath:@"DSC03555.w"
								ofType:@"jpg"];

	if (textures[6]) [self initTexture:textures[6]
								sizedS:1024
								   byT:1024
					  forPictureAtPath:@"M101"
								ofType:@"jpg"];

	if (textures[7]) [self initTexture:textures[7]
								sizedS:1024
								   byT:1024
					  forPictureAtPath:@"land_shallow_topo_2048"
								ofType:@"jpg"];

	if (textures[8]) [self initTexture:textures[8]
								sizedS:1024
								   byT:1024
					  forPictureAtPath:@"Lat Lon"
								ofType:@"jpg"];

	if (textures[9]) [self initTexture:textures[9]
								sizedS:2048
								   byT:1024
					  forPictureAtPath:@"texture-coord-sphere"
								ofType:@"jpg"];

	inited = YES;
	[self reshape];
	
}

- (void)initTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
   forPictureAtPath:(NSString *)path
			 ofType:(NSString *)type
{
	NSBundle *mainBundle = [NSBundle mainBundle];
	NSString *fullPath = [mainBundle pathForResource:path ofType:type];
	NSData *data = [NSData dataWithContentsOfFile:fullPath];
	if (data)
	{
		NSBitmapImageRep *imageRep = [NSBitmapImageRep imageRepWithData:data];
		
		NSString *colorspaceName =			[imageRep colorSpaceName];
		NSBitmapFormat bitmapFormat =		[imageRep bitmapFormat];
		int pixelsWide =					[imageRep pixelsWide];
		int pixelsHigh =					[imageRep pixelsHigh];
		int numberOfPlanes =				[imageRep numberOfPlanes];
		int bytesPerPlane =					[imageRep bytesPerPlane];
		int bytesPerRow =					[imageRep bytesPerRow];
		int bitsPerPixel =					[imageRep bitsPerPixel];
		int samplesPerPixel =				[imageRep samplesPerPixel];
		int bitsPerSample =					[imageRep bitsPerSample];
		int calculatedBytesPerRow =			bitsPerSample * samplesPerPixel * pixelsWide / 8;
		int pad =							bytesPerRow - calculatedBytesPerRow;
		BOOL hasAlpha =						[imageRep hasAlpha];
		unsigned char *bitmapData =			[imageRep bitmapData];
		
		unsigned char *sourceBitmapData = bitmapData;
		unsigned char *newSourceBitmapData = NULL;
		
		size_t bufferSize = s * t * (hasAlpha ? 4 : 3);
		unsigned char *buffer = calloc(bufferSize, 1);
		
		if (pad)
		{
			// create new bitmap buffer and copy rows without pad to new buffer
			
			size_t newSourceBitmapDataSize = calculatedBytesPerRow * pixelsHigh;
			newSourceBitmapData = calloc(newSourceBitmapDataSize, 1);
			unsigned char *sPtr, *dPtr;
			sPtr = bitmapData;
			dPtr = newSourceBitmapData;
			unsigned int i;
			for(i = 0; i < pixelsHigh; i++)
			{
				memcpy(dPtr, sPtr, calculatedBytesPerRow);
				sPtr += bytesPerRow;
				dPtr += calculatedBytesPerRow;
			}
			sourceBitmapData = newSourceBitmapData;
		}
		
		// Images need vertical flipping before use in a MacOS X view. Most of the computer world
		// places the origin at the top left of the image. MacOS X places the origin at the
		// bottom left of the image. Sometimes the flip can happen for free with intelligent 
		// coordinate transformations
		
		flipImage(sourceBitmapData, pixelsHigh, calculatedBytesPerRow);
		
		
		gluScaleImage(internalFormatForColorspace(colorspaceName, hasAlpha),
					  pixelsWide,
					  pixelsHigh,
					  pixelTypeForBPS(bitsPerSample),
					  sourceBitmapData,
					  s,
					  t,
					  pixelTypeForBPS(bitsPerSample),
					  buffer);
		
		/*
		 */
		[self textureInitCommonForTexture:_texture
									level:0
						   internalformat:internalFormatForColorspace(colorspaceName, hasAlpha)
									width:s
								   height:t
								   border:0
								   format:formatForColorspace(colorspaceName, hasAlpha)
									 type:pixelTypeForBPS(bitsPerSample)
								   pixels:buffer];
		
		/*
		glBindTexture (GL_TEXTURE_2D, _texture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		
		glTexImage2D(GL_TEXTURE_2D,
					 0, 
					 internalFormatForColorspace(colorspaceName, hasAlpha), 
					 s,
					 t,
					 0,
					 formatForColorspace(colorspaceName, hasAlpha), 
					 pixelTypeForBPS(bitsPerSample),
					 buffer);
		 */
		
		
		free(buffer);
		if (newSourceBitmapData) free(newSourceBitmapData);
	}
}


- (void)initCheckeredTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
					colorOne:(NSColor *)colorOne
					colorTwo:(NSColor *)colorTwo
			horizontalBlocks:(int)horizontalBlocks
			  verticalBlocks:(int)verticalBlocks
{
	size_t bufferSize = s * t * 4;
	unsigned char *buffer = calloc(bufferSize, 1);
	char c1Red, c1Green, c1Blue, c1Alpa, c2Red, c2Green, c2Blue, c2Alpha;
	
	c1Red =		255.0 * [colorOne redComponent];
	c1Green =	255.0 * [colorOne greenComponent];
	c1Blue =	255.0 * [colorOne blueComponent];
	c1Alpa =	255.0 * [colorOne alphaComponent];
	
	c2Red =		255.0 * [colorTwo redComponent];
	c2Green =	255.0 * [colorTwo greenComponent];
	c2Blue =	255.0 * [colorTwo blueComponent];
	c2Alpha =	255.0 * [colorTwo alphaComponent];
	
	int colorOneBlockWidth = s / horizontalBlocks / 2;
	int colorTwoBlockWidth = colorOneBlockWidth;
	
	int colorOneBlockHeight = t / verticalBlocks / 2;
	
	int l, m, n, o, p;
	
	// l counts horizontal blocks
	// m counts textels within one horizontal row
	// n counts vertical blocks
	// o counts rows within one vertical block
	
	// p indexes the buffer
	
	p = 0;
	
	n = 0;
	while (n < verticalBlocks * 2)
	{
		for (o = 0; o < colorOneBlockHeight; o++)
		{
			l = 0;
			while (l < horizontalBlocks)
			{
				
				if (fmod(n, 2.0))
				{
					for (m = 0; m < colorOneBlockWidth; m++)
					{
						addColorToBufferAtIndex(c1Red, c1Green, c1Blue, c1Alpa, buffer, &p);
					}
					
					for (m = 0; m < colorTwoBlockWidth; m++)
					{
						addColorToBufferAtIndex(c2Red, c2Green, c2Blue, c2Alpha, buffer, &p);
					}
					l++;
				}
				else
				{
					for (m = 0; m < colorOneBlockWidth; m++)
					{
						addColorToBufferAtIndex(c2Red, c2Green, c2Blue, c2Alpha, buffer, &p);
					}
					
					for (m = 0; m < colorTwoBlockWidth; m++)
					{
						addColorToBufferAtIndex(c1Red, c1Green, c1Blue, c1Alpa, buffer, &p);
					}
					l++;
				}
			}
		}
		n++;
	}
	
	[self textureInitCommonForTexture:_texture
								level:0
					   internalformat:GL_RGBA
								width:s
							   height:s
							   border:0
							   format:GL_RGBA
								 type:GL_UNSIGNED_BYTE
							   pixels:buffer];
	
	
	
	free(buffer);
}


- (void)initStripedTexture:(GLuint)_texture
			 sizedS:(GLsizei)s
				byT:(GLsizei)t
					colorOne:(NSColor *)colorOne
					colorTwo:(NSColor *)colorTwo
				   stripes:(int)stripes
{
	size_t bufferSize = s * t * 4;
	unsigned char *buffer = calloc(bufferSize, 1);
	char c1Red, c1Green, c1Blue, c1Alpa, c2Red, c2Green, c2Blue, c2Alpha;
	
	c1Red =		255.0 * [colorOne redComponent];
	c1Green =	255.0 * [colorOne greenComponent];
	c1Blue =	255.0 * [colorOne blueComponent];
	c1Alpa =	255.0 * [colorOne alphaComponent];
	
	c2Red =		255.0 * [colorTwo redComponent];
	c2Green =	255.0 * [colorTwo greenComponent];
	c2Blue =	255.0 * [colorTwo blueComponent];
	c2Alpha =	255.0 * [colorTwo alphaComponent];
	
	int stripeWidth = s / stripes;
	
	int l, m, o, p;
	
	// l counts stripes
	// m counts textels within one horizontal row
	// o counts rows
	
	// p indexes the buffer
	
	p = 0;
	
	for (o = 0; o < t; o++)
	{
		l = 0;
		while (l < stripes / 2)
		{
			for (m = 0; m < stripeWidth; m++)
			{
				addColorToBufferAtIndex(c1Red, c1Green, c1Blue, c1Alpa, buffer, &p);
			}
			
			for (m = 0; m < stripeWidth; m++)
			{
				addColorToBufferAtIndex(c2Red, c2Green, c2Blue, c2Alpha, buffer, &p);
			}
			l++;
		}
	}
	
	[self textureInitCommonForTexture:_texture
								level:0
					   internalformat:GL_RGBA
								width:s
							   height:s
							   border:0
							   format:GL_RGBA
								 type:GL_UNSIGNED_BYTE
							   pixels:buffer];
	
	free(buffer);
}


- (void)textureInitCommonForTexture:(GLuint)_texture
							  level:(GLint)level
					 internalformat:(GLint)internalformat
							  width:(GLsizei)_width
							 height:(GLsizei)_height
							 border:(GLint)border
							 format:(GLenum)format
							   type:(GLenum)type
							 pixels:(const GLvoid *)pixels
{
	glBindTexture (GL_TEXTURE_2D, _texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	// glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	/*
	 */
	
	glTexImage2D(GL_TEXTURE_2D,
				 level, 
				 internalformat, 
				 _width,
				 _height,
				 border,
				 format, 
				 type,
				 pixels);
}


#pragma mark drawing



- (void)drawRect:(NSRect)rect
{
	// printf("-[%s drawRect:]\n", [[self description] UTF8String]);
	
	if (!inited) [self delayedInit];
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	[self setupInitialCoordinateSystem];
	if (drawAxes) [self drawAxes];
	[self drawQuadric];
	
	[self setupLabelCoordinates];
	[self drawVariablesLabels];
	
	[[self openGLContext] flushBuffer];
}

/*
gluCylinder, gluDeleteQuadric, gluDisk, gluPartialDisk, gluQuadricCallback, gluQuadricDrawStyle, gluQuadricNormals, gluQuadricOrientation, gluQuadricTexture, gluSphere 
 */


- (void)drawQuadric
{
	quad = gluNewQuadric();
	if (quad)
	{
		// cull face
		if (cullFace)
		{
			glEnable(GL_CULL_FACE);
			if (cullFace == doGL_FRONT)				glCullFace(GL_FRONT);
			if (cullFace == doGL_BACK)				glCullFace(GL_BACK);
			if (cullFace == doGL_FRONT_AND_BACK)	glCullFace(GL_FRONT_AND_BACK);
		}
		
		// draw style
		gluQuadricDrawStyle(quad, drawStyle);
		
		// normal and orientation
		if (quadricNormal == useGLU_NONE)	gluQuadricNormals(quad, GLU_NONE);
		if (quadricNormal == useGLU_FLAT)	gluQuadricNormals(quad, GLU_FLAT);
		if (quadricNormal == useGLU_SMOOTH) gluQuadricNormals(quad, GLU_SMOOTH);
		
		if (quadricOrientation == useGLU_OUTSIDE)	gluQuadricOrientation(quad, GLU_OUTSIDE);
		if (quadricOrientation == useGLU_INSIDE)	gluQuadricOrientation(quad, GLU_INSIDE);
		
		// texture
		glBindTexture (GL_TEXTURE_2D, textures[currentTexture]);
		gluQuadricTexture(quad, GL_TRUE);
		
		glColor4d(1.0, 1.0, 1.0, 1.0);
		if (quadric == drawGluCylinder)			[self drawCylinder];
		if (quadric == drawGluDisk)				[self drawDisk];
		if (quadric == drawGluPartialDisk)		[self drawPartialDisk];
		if (quadric == drawGluSphere)			[self drawSphere];
		
		if (cullFace) glDisable(GL_CULL_FACE);
		
		gluDeleteQuadric(quad);
	}
	else
	{
		printf("quad allocation failed\n");
	}
}


- (void)drawCylinder
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	// glRotated(180, 1.0, 1.0, 0.0);
	glRotated(zRotation, 0.0, 0.0, 1.0);		
	
	gluCylinder(quad, base,  top, height, slices, stacks);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}


- (void)drawDisk
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	// glRotated(180.0, 0.0, 1.0, 0.0);	// 180 about y axis
	glRotated(zRotation, 0.0, 0.0, 1.0);		//  45 about z axis
	
	gluDisk(quad, inner, outer, slices, loops);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}


- (void)drawPartialDisk
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	// glRotated(180.0, 0.0, 1.0, 0.0);	// 180 about y axis
	glRotated(zRotation, 0.0, 0.0, 1.0);		//  45 about z axis
	
	gluPartialDisk(quad, inner, outer, slices, loops, start, sweep);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}


- (void)drawSphere
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	
	// glRotated(180.0, 1.0, 1.0, 1.0);
	glRotated(zRotation, 0.0, 0.0, 1.0);		
	
	// glEnable(GL_CULL_FACE);
	// glCullFace(GL_FRONT);
	
	gluSphere(quad, radius, slices, stacks);
	
	// glDisable(GL_CULL_FACE);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}


- (void)drawTextureInRectangle
{
	
}


- (void)drawVariablesLabels
{
	[self drawCullFaceLabel];
	[self drawDrawStyleLabel];
	[self drawNormalsLabel];
	[self drawOrientationLabel];
	[self drawDepthTestEnabledLabel];
	[self drawQuadricLabel];
	[self drawTextureLabel];
	[self drawZRotationLabel];
}


- (void)drawCullFaceLabel
{
	[stringRenderer renderString:[self cullFaceLabel]
							 atX:5.0
							   y:95.0
							   z:0.0];
}


- (void)drawDrawStyleLabel
{
	[stringRenderer renderString:[self drawStyleLabel]
							 atX:5.0
							   y:5.0
							   z:0.0];
}


- (void)drawNormalsLabel
{
	
}


- (void)drawOrientationLabel
{
	
}


- (void)drawDepthTestEnabledLabel
{
	[stringRenderer renderString:[self depthTestEnabledLabel]
							 atX:5.0
							   y:15.0
							   z:0.0];
}


- (void)drawQuadricLabel
{
	
}


- (void)drawTextureLabel
{
	[stringRenderer renderString:[self textureLabel]
							 atX:50.0
							   y:5.0
							   z:0.0];
}


- (void)drawZRotationLabel
{
	[stringRenderer renderString:[self zRotationLabel]
							 atX:5.0
							   y:10.0
							   z:0.0];
}


- (void)setupInitialCoordinateSystem
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrthoFactor = 10000.0;		//	10,000.0
	
	/*
	minXOrtho =  glOrthoFactor;		// displays sphere with map projected correctly
	maxXOrtho = -glOrthoFactor;		// but pictures on disk are upside down
	minYOrtho =  glOrthoFactor;		// also, if sphere is used in other apps
	maxYOrtho = -glOrthoFactor;		// reversed coordinate system will cause
	minZOrtho = -glOrthoFactor;		// problems in those apps
	maxZOrtho =  glOrthoFactor;
	 */
	
	minXOrtho = -glOrthoFactor;
	maxXOrtho =  glOrthoFactor;
	minYOrtho = -glOrthoFactor;
	maxYOrtho =  glOrthoFactor;
	minZOrtho = -glOrthoFactor;
	maxZOrtho =  glOrthoFactor;
	
	glOrtho(minXOrtho, maxXOrtho, 
			minYOrtho, maxYOrtho, 
			minZOrtho, maxZOrtho);
	
	

	eyeX = 500.0;
	eyeY = 500.0;
	eyeZ = 500.0;
	centerX = 0.0;
	centerY = 0.0;
	centerZ = 0.0;
	upX = 0.0;
	upY = 0.0;
	upZ = 1.0;
	
	/*
	eyeX = 1.0;
	eyeY = 1.0;
	eyeZ = 1.0;
	centerX = 0.0;
	centerY = 0.0;
	centerZ = 0.0;
	upX = 0.0;
	upY = 0.0;
	upZ = 1.0;
	 */
	
	
	
	gluLookAt(eyeX,
			  eyeY,
			  eyeZ,
			  centerX,
			  centerY,
			  centerZ,
			  upX,
			  upY,
			  upZ);
		
	glMatrixMode(GL_MODELVIEW);
}


- (void)setupLabelCoordinates
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	GLdouble aminXOrtho, amaxXOrtho, aminYOrtho, amaxYOrtho, aminZOrtho, amaxZOrtho;
	GLdouble aglOrthoFactor = 100.0;
	aminXOrtho = 0.0;
	amaxXOrtho = aglOrthoFactor;
	aminYOrtho = 0.0;
	amaxYOrtho = aglOrthoFactor;
	aminZOrtho = 0.0;
	amaxZOrtho = aglOrthoFactor;
	
	glOrtho(aminXOrtho, amaxXOrtho, 
			aminYOrtho, amaxYOrtho, 
			aminZOrtho, amaxZOrtho);
		
	glMatrixMode(GL_MODELVIEW);
}


- (void)drawAxes
{
	// x axis in red
	
	glBegin(GL_LINES);
		glColor4d(0.3, 0.0, 0.0, 1.0);				// dark red
		glVertex3d(minXOrtho, 0.0, 0.0);
		glColor4d(1.0, 0.0, 0.0, 1.0);				// bright red
		glVertex3d(maxXOrtho, 0.0, 0.0);
	glEnd();
	
	[stringRenderer renderString:"I"
							 atX:maxXOrtho * 1.1
							   y:0.0
							   z:0.0];
	
	
	// y axis in green
	
	glBegin(GL_LINES);
		glColor4d(0.0, 0.3, 0.0, 1.0);				// dark green
		glVertex3d(0.0, minYOrtho, 0.0);
		glColor4d(0.0, 1.0, 0.0, 1.0);				// bright green
		glVertex3d(0.0, maxYOrtho, 0.0);
	glEnd();
	
	[stringRenderer renderString:"J"
							 atX:0.0
							   y:maxYOrtho * 1.1
							   z:0.0];
	
	// z axis in blue
	
	glBegin(GL_LINES);
		glColor4d(0.0, 0.0, 0.3, 1.0);				// dark blue
		glVertex3d(0.0, 0.0, minZOrtho);
		glColor4d(0.0, 0.0, 1.0, 1.0);				// bright blue
		glVertex3d(0.0, 0.0, maxZOrtho);
	glEnd();
	
	[stringRenderer renderString:"K"
							 atX:0.0
							   y:0.0
							   z:maxZOrtho * 1.1];
}


- (void) reshape
{
    if(inited)
    {
		[[self openGLContext] makeCurrentContext];
		
		NSRect bounds = [self bounds];
		glViewport(0, 0, bounds.size.width, bounds.size.height);
		
		[self setNeedsDisplay:YES];
    }
}




#pragma mark labels


- (char *)cullFaceLabel
{
	NSString *cullFaceStr =  @"";
	if (cullFace == disableCullFAce)		cullFaceStr = @"disabled";
	if (cullFace == doGL_FRONT)				cullFaceStr = @"GL_FRONT";
	if (cullFace == doGL_BACK)				cullFaceStr = @"GL_BACK";
	if (cullFace == doGL_FRONT_AND_BACK)	cullFaceStr = @"GL_FRONT_AND_BACK";
	
	NSString *cullFaceLabel = [NSString stringWithFormat:@"facet culling: %@", cullFaceStr];
	return (char *)[cullFaceLabel UTF8String];
}


- (char *)drawStyleLabel
{
	NSString *drawStyleStr =  @"";
	if (drawStyle == GLU_POINT)			drawStyleStr =  @"GLU_POINT";
	if (drawStyle == GLU_LINE)			drawStyleStr =  @"GLU_LINE";
	if (drawStyle == GLU_FILL)			drawStyleStr =  @"GLU_FILL";
	if (drawStyle == GLU_SILHOUETTE)	drawStyleStr =  @"GLU_SILHOUETTE";
	
	NSString *drawStyleLabel = [NSString stringWithFormat:@"drawStyle = %@", drawStyleStr];
	return (char *)[drawStyleLabel UTF8String];
}


- (char *)depthTestEnabledLabel
{
	NSString *depthTestEnabledLabel = [NSString stringWithFormat:@"depthTestEnabled = %@", depthTestEnabled ? @"YES" : @"NO"];
	return (char *)[depthTestEnabledLabel UTF8String];
}


- (char *)textureLabel
{
	NSString *textureLabel = [NSString stringWithFormat:@"texture = %@", [textureLabelsArray objectAtIndex:currentTexture]];
	return (char *)[textureLabel UTF8String];
}


- (char *)zRotationLabel
{
	NSString *zRotationLabel = [NSString stringWithFormat:@"zRotation = %g", zRotation];
	return (char *)[zRotationLabel UTF8String];
}



#pragma mark control

- (void)updateCullFace:(NSEvent *)theEvent
{
	cullFace++;
	if (cullFace > doGL_FRONT_AND_BACK) cullFace = disableCullFAce;
	[self setNeedsDisplay:YES];
}


- (void)updateDrawStyle:(NSEvent *)theEvent
{
	drawStyle++;
	if (drawStyle > GLU_SILHOUETTE) drawStyle = GLU_POINT;
	[self setNeedsDisplay:YES];
}


- (void)updateNormals:(NSEvent *)theEvent
{
	quadricNormal++;
	if (quadricNormal > useGLU_SMOOTH) quadricNormal = useGLU_NONE;
	[self setNeedsDisplay:YES];
}


- (void)updateOrientation:(NSEvent *)theEvent
{
	quadricOrientation++;
	if (quadricOrientation > useGLU_INSIDE) quadricOrientation = useGLU_OUTSIDE;
	[self setNeedsDisplay:YES];
}


- (void)updateQuadric:(NSEvent *)theEvent
{
	quadric++;
	if (quadric > drawGluSphere) quadric = drawGluCylinder;
	[self setNeedsDisplay:YES];
}


- (void)updateBackground:(NSEvent *)theEvent
{
	background++;
	if (background > whiteBackground) background = blackBackground;
	setClearColorForBackground(background);
	[self setNeedsDisplay:YES];
}


- (void)updateCurrentTexture:(NSEvent *)theEvent
{
	currentTexture++;
	if (currentTexture == kTextureCount) currentTexture = 0;
	[self setNeedsDisplay:YES];
}


- (void)startZRotationTimer
{
	[zRotationTimer invalidate];
	zRotationTimer = [NSTimer scheduledTimerWithTimeInterval:zRotationInterval
													  target:self
													selector:@selector(animateZRotation:)
													userInfo:nil
													 repeats:YES];
}


- (void)stopZRotationTimer
{
	if (zRotationTimer) [zRotationTimer invalidate];
	zRotationTimer = nil;
}


- (void)animateZRotation:(NSTimer *)timer
{
	zRotation += dZRotation;
	if (zRotation > 360.0) zRotation -= 360.0;
	[self setNeedsDisplay:YES];
}


- (void)toggleZRotationAnimation:(NSEvent *)theEvent
{
	if (zRotationTimer)
	{
		[self stopZRotationTimer];
	}
	else
	{
		[self startZRotationTimer];
	}
}



/*
- (void)toggleDepthTestEnabled:(NSEvent *)theEvent
{
	if (glIsEnabled(GL_DEPTH_TEST))
	{
		glDisable(GL_DEPTH_TEST);
		depthTestEnabled = NO;
	}
	else
	{
		glEnable(GL_DEPTH_TEST);
		depthTestEnabled = YES;
	}
	[self setNeedsDisplay:YES];
}
 */


- (void)toggleDepthTestEnabled:(NSEvent *)theEvent
{
	depthTestEnabled = !depthTestEnabled;
	if (depthTestEnabled)
	{
		glEnable(GL_DEPTH_TEST);
	}
	else
	{
		glDisable(GL_DEPTH_TEST);
	}
	[self setNeedsDisplay:YES];
}


- (void)toggleDrawAxes:(NSEvent *)theEvent
{
	drawAxes = !drawAxes;
	[self setNeedsDisplay:YES];
}


- (BOOL)acceptsFirstResponder
{
	return YES;
}


- (void)setFlagsfromEvent:(NSEvent *)theEvent
{
	unsigned int modifiers = [theEvent modifierFlags];
		
	optionKeyDown =			NSAlternateKeyMask ==	(modifiers & NSAlternateKeyMask);
	controlKeyDown =		NSControlKeyMask ==		(modifiers & NSControlKeyMask);
	commandKeyDown =		NSCommandKeyMask ==		(modifiers & NSCommandKeyMask);
	shiftDown =				NSShiftKeyMask ==		(modifiers & NSShiftKeyMask);
	capsLockDown =			NSAlphaShiftKeyMask ==	(modifiers & NSAlphaShiftKeyMask);
}
// glutSphere

- (void)flagsChanged:(NSEvent *)theEvent
{
	[self setFlagsfromEvent:theEvent];
}


- (void)keyDown:(NSEvent *)theEvent
{
	NSString *characters = [theEvent characters];
	unichar firstCharacter = [characters characterAtIndex:0];
	
	if (firstCharacter == 'a') [self toggleDrawAxes:theEvent];						// draw axes
	if (firstCharacter == 'b') [self updateBackground:theEvent];					// next background color
	if (firstCharacter == 'c') [self updateCullFace:theEvent];						// next cull face
	if (firstCharacter == 'd') [self updateDrawStyle:theEvent];						// next draw style
	if (firstCharacter == 'n') [self updateNormals:theEvent];						// next normal choice
	if (firstCharacter == 'o') [self updateOrientation:theEvent];					// next orientation
	if (firstCharacter == 'p') [self toggleDepthTestEnabled:theEvent];				// enable / disable depth testing
	if (firstCharacter == 'q') [self updateQuadric:theEvent];						// next quadric
	if (firstCharacter == 't') [self updateCurrentTexture:theEvent];				// next texture
	if (firstCharacter == 'z') [self toggleZRotationAnimation:theEvent];			// z rotation on / off
}



#pragma mark accessors


- (void)setBase:(GLdouble)_base
{
	base = _base;
}


- (void)setTop:(GLdouble)_top
{
	top = _top;
}


- (void)setHeight:(GLdouble)_height
{
	height = _height;
}


- (void)setRadius:(GLdouble)_radius
{
	radius = _radius;
}


- (void)setInner:(GLdouble)_inner
{
	inner = _inner;
}


- (void)setOuter:(GLdouble)_outer
{
	outer = _outer;
}


- (void)setStart:(GLdouble)_start
{
	start = _start;
}


- (void)setSweep:(GLdouble)_sweep
{
	sweep = _sweep;
}


- (void)setLoops:(GLint)_loops
{
	loops = _loops;
}


- (void)setSlices:(GLint)_slices
{
	slices = _slices;
}


- (void)setStacks:(GLint)_stacks
{
	stacks = _stacks;
}



@end


/*
Any idea why this works:

  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, iwidth, iheight, GL_RGB,
GL_UNSIGNED_BYTE, image);

but this doesn't:

  glTexImage2D(GL_TEXTURE_2D, 0, 3, iwidth, iheight, 0, GL_RGB,
GL_UNSIGNED_BYTE, image);

The latter acts as if texture mapping were disabled.

Thanks,

-- 
John Brewer

_______________________________________________
Opengl-students mailing list
Opengl-students@bignerdranch.com
http://mail.bignerdranch.com/mailman/listinfo/opengl-students_bignerdranch.com
 
 */
