//
//  StringRenderer.m
//  gluNewQuadric
//
//  Created by Tom Bernard on 1/19/08.
//  Copyright 2008 Bersearch Information Services. All rights reserved.
//

#import "StringRenderer.h"


@implementation StringRenderer


- (id)initWithView:(NSOpenGLView *)_view
{
	if (self = [super init])
	{
		view = _view;
		
		red = 1.0;		// default color light yellow
		green = 1.0;
		blue = 0.5;
		alpha = 1.0;
		
		balpha = 0.5;	// default background black with 50% alpha
	}
	return self;
}


- (void)setColor:(NSColor *)color
{
	red =		[color redComponent];
	green =		[color greenComponent];
	blue =		[color blueComponent];
	alpha =		[color alphaComponent];
}


- (void)setBackgroundColor:(NSColor *)color
{
	bred =		[color redComponent];
	bgreen =	[color greenComponent];
	bblue =		[color blueComponent];
	balpha =	[color alphaComponent];
}


- (void)renderString:(char *)string
				 atX:(GLfloat)_x 
				   y:(GLfloat)_y 
				   z:(GLfloat)_z
{
	glRasterPos3f(_x, _y, _z);
	
	[self renderBackgoundBoxForString:string
								  atX:_x 
									y:_y 
									z:_z];
	
	glColor4f(red, green, blue, alpha);
	while (*string)
	{
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *string);
		string++;
	}
}


- (void)renderBackgoundBoxForString:(char *)string
								atX:(GLfloat)_x 
								  y:(GLfloat)_y 
								  z:(GLfloat)_z
{
	GLfloat bbllx, bblly, bblrx, bblry, bburx, bbury, bbulx, bbuly;
	
	GLfloat border, heightWithBorder;
	
	NSRect frame = [view frame];
	heightWithBorder = 16.0 / frame.size.height;
	border = 2.0 / frame.size.height;
		
		
	bbllx = _x - border;
	bblly = _y - border;
	
	bbulx = bbllx;
	bbuly = bblly + heightWithBorder;
	
	GLfloat width;
	width = [self stringwidthForString:string];
	width = width / frame.size.width;
	
	bblrx = bbllx + width + border + border;
	bblry = bblly;
	
	bburx = bblrx;
	bbury = bbuly;
	
	glBegin(GL_QUADS);
	glColor4f(bred, bgreen, bblue, balpha);
	glVertex2f(bbllx, bblly);
	glVertex2f(bblrx, bblry);
	glVertex2f(bburx, bbury);
	glVertex2f(bbulx, bbuly);
	glEnd();
	
}


- (float)stringwidthForString:(char *)_string
{
	float width = 0.0;
	
	while (*_string)
	{
		width += glutBitmapWidth(GLUT_BITMAP_HELVETICA_12, *_string);
		_string++;
	}
	
	return width;
}


@end
